@@ -4,7 +4,7 @@
 // Boring, average every day contemporary imports
 use std::{
 	fs::File,
-	io::{Seek, SeekFrom, Read, Cursor},
+	io::{Read, Cursor},
 	str,
 };
 
@@ -12,7 +12,8 @@ use crate::{global::result::InternalResult, prelude::*};
 
 // Contains both the public key and secret key in the same file:
 // secret -> [u8; crate::SECRET_KEY_LENGTH], public -> [u8; crate::PUBLIC_KEY_LENGTH]
-const KEYPAIR: &str = "test_data/pair.pub";
+const KEYPAIR_PATH: &str = "test_data/pair.pub";
+const KEYPAIR: &[u8; crate::KEYPAIR_LENGTH] = include_bytes!("../../test_data/pair.pub");
 
 // The paths to the Archives, to be written|loaded
 const SIGNED_TARGET: &str = "test_data/signed/target.vach";
@@ -97,10 +98,13 @@ fn header_config() -> InternalResult<()> {
 	use crate::global::header::Header;
 
 	let config = HeaderConfig::new(*b"VfACH", None);
-	let mut file = File::open("test_data/simple/target.vach")?.take(Header::BASE_SIZE as u64);
 	println!("{}", &config);
 
-	let header = Header::from_handle(&mut file)?;
+	let mut header_data = [0u8; Header::BASE_SIZE];
+	let mut file = File::open("test_data/simple/target.vach")?;
+	file.read(&mut header_data)?;
+
+	let header = Header::from_handle(header_data.as_slice())?;
 	println!("{}", header);
 
 	Header::validate(&header, &config)?;
@@ -110,9 +114,7 @@ fn header_config() -> InternalResult<()> {
 #[test]
 fn builder_no_signature() -> InternalResult<()> {
 	let mut builder = Builder::default();
-	let build_config = BuilderConfig::default().callback(|id, _, entry| {
-		dbg!(id, entry);
-	});
+	let build_config = BuilderConfig::default();
 
 	builder.add(File::open("test_data/song.txt")?, "song")?;
 	builder.add(File::open("test_data/lorem.txt")?, "lorem")?;
@@ -120,10 +122,7 @@ fn builder_no_signature() -> InternalResult<()> {
 	builder.add(File::open("test_data/quicksort.wasm")?, "wasm")?;
 
 	let mut poem_flags = Flags::default();
-	poem_flags.set(
-		CUSTOM_FLAG_1 | CUSTOM_FLAG_2 | CUSTOM_FLAG_3 | CUSTOM_FLAG_4,
-		true,
-	)?;
+	poem_flags.set(CUSTOM_FLAG_1 | CUSTOM_FLAG_2 | CUSTOM_FLAG_3 | CUSTOM_FLAG_4, true)?;
 
 	builder.add_leaf(
 		Leaf::from_handle(File::open("test_data/poem.txt")?)
@@ -148,7 +147,7 @@ fn builder_no_signature() -> InternalResult<()> {
 #[test]
 fn fetch_no_signature() -> InternalResult<()> {
 	let target = File::open(SIMPLE_TARGET)?;
-	let mut archive = Archive::from_handle(target)?;
+	let archive = Archive::from_handle(target)?;
 	dbg!(archive.entries());
 	let resource = archive.fetch("poem")?;
 
@@ -184,7 +183,7 @@ fn gen_keypair() -> InternalResult<()> {
 	if regenerate {
 		let keypair = gen_keypair();
 
-		std::fs::write(KEYPAIR, &keypair.to_bytes())?;
+		std::fs::write(KEYPAIR_PATH, &keypair.to_bytes())?;
 	};
 
 	Ok(())
@@ -194,10 +193,12 @@ fn gen_keypair() -> InternalResult<()> {
 fn builder_with_signature() -> InternalResult<()> {
 	let mut builder = Builder::default();
 
-	let mut build_config = BuilderConfig::default().callback(|_, _, d| {
-		dbg!(&d);
-	});
-	build_config.load_keypair(File::open(KEYPAIR)?)?;
+	let cb = |_: &str, entry: &RegistryEntry| {
+		dbg!(entry);
+	};
+	let mut build_config = BuilderConfig::default().callback(&cb);
+
+	build_config.load_keypair(KEYPAIR.as_slice())?;
 
 	builder.add_dir(
 		"test_data",
@@ -222,11 +223,10 @@ fn fetch_with_signature() -> InternalResult<()> {
 
 	// Load keypair
 	let mut config = HeaderConfig::default();
-	let mut keypair = File::open(KEYPAIR)?;
-	keypair.seek(SeekFrom::Start(crate::SECRET_KEY_LENGTH as u64))?;
+	let keypair = &KEYPAIR[crate::SECRET_KEY_LENGTH..];
 	config.load_public_key(keypair)?;
 
-	let mut archive = Archive::with_config(target, &config)?;
+	let archive = Archive::with_config(target, &config)?;
 	let resource = archive.fetch("test_data/song.txt")?;
 	let song = str::from_utf8(resource.data.as_slice()).unwrap();
 
@@ -240,7 +240,7 @@ fn fetch_with_signature() -> InternalResult<()> {
 	assert!(!not_signed_resource.flags.contains(Flags::SIGNED_FLAG));
 	assert!(!not_signed_resource.secured);
 
-	// Check identity of retrieved data
+	// Check authenticity of retrieved data
 	println!("{}", song);
 
 	// Windows bullshit
@@ -265,11 +265,10 @@ fn fetch_write_with_signature() -> InternalResult<()> {
 
 	// Load keypair
 	let mut config = HeaderConfig::default();
-	let mut keypair = File::open(KEYPAIR)?;
-	keypair.seek(SeekFrom::Start(crate::SECRET_KEY_LENGTH as u64))?;
+	let keypair = &KEYPAIR[crate::SECRET_KEY_LENGTH..];
 	config.load_public_key(keypair)?;
 
-	let mut archive = Archive::with_config(target, &config)?;
+	let archive = Archive::with_config(target, &config)?;
 	let mut song = Vec::new();
 
 	let metadata = archive.fetch_write("test_data/poem.txt", &mut song)?;
@@ -295,12 +294,11 @@ fn fetch_write_with_signature() -> InternalResult<()> {
 #[test]
 fn edcryptor_test() -> InternalResult<()> {
 	use crate::utils::gen_keypair;
-	use crate::global::edcryptor::EDCryptor;
+	use crate::global::edcryptor::Encryptor;
 
 	let pk = gen_keypair().public;
 
-	let crypt = EDCryptor::new(&pk, crate::DEFAULT_MAGIC.clone());
-
+	let crypt = Encryptor::new(&pk, crate::DEFAULT_MAGIC.clone());
 	let data = vec![12, 12, 12, 12];
 
 	let ciphertext = crypt.encrypt(&data).unwrap();
@@ -313,15 +311,10 @@ fn edcryptor_test() -> InternalResult<()> {
 
 #[test]
 fn builder_with_encryption() -> InternalResult<()> {
-	let mut builder = Builder::new().template(
-		Leaf::default()
-			.encrypt(true)
-			.compress(CompressMode::Never)
-			.sign(true),
-	);
+	let mut builder = Builder::new().template(Leaf::default().encrypt(true).compress(CompressMode::Never).sign(true));
 
 	let mut build_config = BuilderConfig::default();
-	build_config.load_keypair(File::open(KEYPAIR)?)?;
+	build_config.load_keypair(KEYPAIR.as_slice())?;
 
 	builder.add_dir("test_data", None)?;
 
@@ -340,16 +333,15 @@ fn fetch_from_encrypted() -> InternalResult<()> {
 
 	// Load keypair
 	let mut config = HeaderConfig::default();
-	let mut public_key = File::open(KEYPAIR)?;
-	public_key.seek(SeekFrom::Start(crate::SECRET_KEY_LENGTH as u64))?;
+	let public_key = &KEYPAIR[crate::SECRET_KEY_LENGTH..];
 	config.load_public_key(public_key)?;
 
-	let mut archive = Archive::with_config(target, &config)?;
+	let archive = Archive::with_config(target, &config)?;
 	let resource = archive.fetch("test_data/song.txt")?;
 	let song = str::from_utf8(resource.data.as_slice()).unwrap();
 
-	// Check identity of retrieved data
-	println!("{}", song);
+	// Log archive to console
+	dbg!(&archive);
 
 	// Windows bullshit
 	#[cfg(target_os = "windows")]
@@ -368,46 +360,6 @@ fn fetch_from_encrypted() -> InternalResult<()> {
 	Ok(())
 }
 
-#[test]
-fn cyclic_linked_leafs() {
-	use std::io::Cursor;
-
-	// init
-	let mut target = Cursor::new(Vec::<u8>::new());
-
-	// Builder stage
-	let mut builder = Builder::default();
-
-	builder
-		.add_leaf(
-			Leaf::default()
-				.id("d2_link")
-				.link_mode(Some("d1_link".to_string())),
-		)
-		.unwrap();
-	builder
-		.add_leaf(
-			Leaf::default()
-				.id("d1_link")
-				.link_mode(Some("d2_link".to_string())),
-		)
-		.unwrap();
-	builder
-		.dump(&mut target, &BuilderConfig::default())
-		.unwrap();
-
-	target.seek(SeekFrom::Start(0)).unwrap();
-	let mut archive = Archive::from_handle(target).unwrap();
-
-	// Assert that this causes an error, [Cyclic Linked Leafs]
-	if let Err(err) = archive.fetch("d1_link") {
-		match err {
-			InternalError::CyclicLinkReferenceError(_, _) => (),
-			_ => panic!("Unrecognized error. Expected cyclic linked leaf error"),
-		}
-	};
-}
-
 #[test]
 fn consolidated_example() -> InternalResult<()> {
 	use crate::utils::{gen_keypair, read_keypair};
@@ -429,25 +381,27 @@ fn consolidated_example() -> InternalResult<()> {
 	let mut builder = Builder::new().template(Leaf::default().encrypt(true));
 
 	// Add data
+	let template = Leaf::default()
+		.encrypt(true)
+		.version(59)
+		.compression_algo(CompressionAlgorithm::Snappy);
 	builder.add_leaf(
 		Leaf::from_handle(data_1)
 			.id("d1")
-			.compress(CompressMode::Always),
+			.compress(CompressMode::Always)
+			.template(&template),
 	)?;
 	builder.add_leaf(
 		Leaf::from_handle(data_2)
 			.id("d2")
-			.compress(CompressMode::Never),
+			.compress(CompressMode::Never)
+			.template(&template),
 	)?;
 	builder.add_leaf(
 		Leaf::from_handle(data_3)
 			.id("d3")
-			.compress(CompressMode::Detect),
-	)?;
-	builder.add_leaf(
-		Leaf::default()
-			.id("d3_link")
-			.link_mode(Some("d3".to_string())),
+			.compress(CompressMode::Detect)
+			.template(&template),
 	)?;
 
 	// Dump data
@@ -457,30 +411,30 @@ fn consolidated_example() -> InternalResult<()> {
 	// Just because
 	println!("Building took: {}us", then.elapsed().as_micros());
 
-	// Ensure your stream_position is where you want it to be, so here at the start of the Cursor (0)
-	target.seek(SeekFrom::Start(0))?;
-
 	// Load data
 	let mut config = HeaderConfig::default().magic(*MAGIC);
 	config.load_public_key(&keypair_bytes[32..])?;
 
-	let mut archive = Archive::with_config(target, &config)?;
+	let then = Instant::now();
+	let archive = Archive::with_config(target, &config)?;
+	dbg!(archive.entries());
+
+	println!("Archive initialization took: {}us", then.elapsed().as_micros());
 
 	// Quick assertions
 	let then = Instant::now();
 	assert_eq!(archive.fetch("d1")?.data.as_slice(), data_1);
 	assert_eq!(archive.fetch("d2")?.data.as_slice(), data_2);
 	assert_eq!(archive.fetch("d3")?.data.as_slice(), data_3);
-	assert_eq!(archive.fetch("d3_link")?.data.as_slice(), data_3);
 
-	println!("Fetching took: {}us", then.elapsed().as_micros());
+	println!("Fetching took: {}us on average", then.elapsed().as_micros() / 4u128);
 
 	// All seems ok
 	Ok(())
 }
 
 #[test]
-fn test_compression() -> InternalResult<()> {
+fn test_compressors() -> InternalResult<()> {
 	const INPUT_LEN: usize = 4096;
 
 	let input = [12u8; INPUT_LEN];
@@ -508,9 +462,7 @@ fn test_compression() -> InternalResult<()> {
 
 	builder.dump(&mut target, &BuilderConfig::default())?;
 
-	target.seek(SeekFrom::Start(0))?;
-
-	let mut archive = Archive::from_handle(&mut target)?;
+	let archive = Archive::from_handle(&mut target)?;
 
 	let d1 = archive.fetch("LZ4")?;
 	let d2 = archive.fetch("BROTLI")?;
@@ -530,5 +482,64 @@ fn test_compression() -> InternalResult<()> {
 	assert!(archive.fetch_entry("BROTLI").unwrap().offset < INPUT_LEN as u64);
 	assert!(archive.fetch_entry("SNAPPY").unwrap().offset < INPUT_LEN as u64);
 
+	// A simple test to show that these are somehow not the same data
+	assert!(archive.fetch_entry("SNAPPY").unwrap().offset != archive.fetch_entry("LZ4").unwrap().offset);
+	assert!(archive.fetch_entry("BROTLI").unwrap().offset != archive.fetch_entry("LZ4").unwrap().offset);
+	assert!(archive.fetch_entry("SNAPPY").unwrap().offset != archive.fetch_entry("BROTLI").unwrap().offset);
+
+	Ok(())
+}
+
+#[test]
+#[cfg(feature = "multithreaded")]
+fn test_batch_fetching() -> InternalResult<()> {
+	// Define input constants
+	const INPUT_LEN: usize = 512;
+	let mut input = [12u8; INPUT_LEN];
+	input.iter_mut().for_each(|i| *i = rand::random::<u8>());
+
+	let mut target = Cursor::new(vec![]);
+	let mut builder = Builder::new();
+
+	// Define and queue data
+	builder.add_leaf(Leaf::from_handle(input.as_slice()).id("LZ4"))?;
+	builder.add_leaf(Leaf::from_handle(input.as_slice()).id("BROTLI"))?;
+	builder.add_leaf(Leaf::from_handle(input.as_slice()).id("SNAPPY"))?;
+
+	// Process data
+	builder.dump(&mut target, &BuilderConfig::default())?;
+
+	let mut archive = Archive::from_handle(&mut target)?;
+	let mut resources = archive.fetch_batch(["LZ4", "BROTLI", "SNAPPY", "NON_EXISTENT"].into_iter())?;
+
+	// Tests and checks
+	assert!(resources.get("NON_EXISTENT").is_some());
+	match resources.get("NON_EXISTENT").unwrap() {
+		Ok(_) => return Err(InternalError::OtherError("This should be an error".to_string())),
+		Err(err) => match err {
+			&InternalError::MissingResourceError(_) => {}
+
+			specific => {
+				return Err(InternalError::OtherError(format!(
+					"Unrecognized error: {}, it should be a `InternalError::MissingResourceError` error",
+					specific
+				)))
+			}
+		},
+	}
+
+	let d1 = resources.remove("LZ4").unwrap().unwrap();
+	let d2 = resources.remove("BROTLI").unwrap().unwrap();
+	let d3 = resources.remove("SNAPPY").unwrap().unwrap();
+
+	// Identity tests
+	assert_eq!(d1.data.len(), INPUT_LEN);
+	assert_eq!(d2.data.len(), INPUT_LEN);
+	assert_eq!(d3.data.len(), INPUT_LEN);
+
+	assert!(&d1.data[..] == &input);
+	assert!(&d2.data[..] == &input);
+	assert!(&d3.data[..] == &input);
+
 	Ok(())
 }
